/* filename:    irfplot.g
** description: The program plots the 
**              the VAR impulse response functions
** created:     11/06/01
*/


/* filename:    irfpmom.g
** description: The program computes posterior mean and confidence bands
**              for the VAR impulse response functions
** created:     11/06/01
*/

new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  _verbose = 1;           /** 1 for verbose **/

#include pathspec.g;

nvar = 4;
nirf = 40;

/******************************************************************
**         Load IRF summaries
*/

ldsgeirf1 = workpath $+
            "\\..\\msearch\\m2ltm21t0p2\\results\\mhrun1\\dsgeirfs";
ldsgeirf2 = workpath $+
            "\\..\\miu\\m2ltm11t0p2\\results\\mhrun1\\dsgeirfs";

ti    = seqa(1,1,nirf);

/* Read Posterior Simulation Summary File:
** DSGE model 1
*/
open fhdsgeirf = ^ldsgeirf1 for read;

dsgeirf1_m = readr(fhdsgeirf,1);
dsgeirf1_s = readr(fhdsgeirf,1);
dsgeirf1_hpd_l = readr(fhdsgeirf,1);
dsgeirf1_hpd_h = readr(fhdsgeirf,1);

dsgeirf1_m     = reshape(dsgeirf1_m,nvar^2,nirf)';
dsgeirf1_hpd_l = reshape(dsgeirf1_hpd_l,nvar^2,nirf)';
dsgeirf1_hpd_h = reshape(dsgeirf1_hpd_h,nvar^2,nirf)';

closeall fhdsgeirf;

/* Read Posterior Simulation Summary File:
** DSGE model 2
*/
open fhdsgeirf = ^ldsgeirf2 for read;

dsgeirf2_m = readr(fhdsgeirf,1);
dsgeirf2_s = readr(fhdsgeirf,1);
dsgeirf2_hpd_l = readr(fhdsgeirf,1);
dsgeirf2_hpd_h = readr(fhdsgeirf,1);

dsgeirf2_m     = reshape(dsgeirf2_m,nvar^2,nirf)';
dsgeirf2_hpd_l = reshape(dsgeirf2_hpd_l,nvar^2,nirf)';
dsgeirf2_hpd_h = reshape(dsgeirf2_hpd_h,nvar^2,nirf)';

closeall fhdsgeirf;

/* Plot Impulse Response functions
*/

titlestr = "GOV -> Real GDP (cyc)" $|
           "GOV -> Inflation"      $|
           "GOV -> FedFUNDS"       $|
           "GOV -> Real M2 (cyc)"  $|
           "CHI -> Real GDP (cyc)" $|
           "CHI -> Inflation"      $|
           "CHI -> FedFUNDS"       $|
           "CHI -> Real M2 (cyc)"  $|
           "R -> Real GDP (cyc)" $|
           "R -> Inflation"      $|
           "R -> FedFUNDS"       $|
           "R -> Real M2 (cyc)"  $|
           "Z -> Real GDP (cyc)" $|
           "Z -> Inflation"      $|
           "Z -> FedFUNDS"       $|
           "Z -> Real M2 (cyc)";

ystr     = "Percent";


drawtics = ( -0.0 ~ 0.75 ~ 0.25 )|
           ( -2.5 ~ 0 ~ 0.5 )|
           ( -1.5 ~ 0 ~ 0.5 )|
           ( -0.0 ~ 1 ~ 0.5 )|
           
           ( -0.25 ~ 0.25 ~ 0.5 )|
           ( -0.25 ~ 0.25 ~ 0.25 )|
           ( -0.15 ~ 0.15 ~ 0.05 )|
           ( -0.25 ~ 0.75 ~ 0.5 )|
           
           ( -0.25 ~ 0.25 ~ 0.25 )|
           ( -1.5 ~ 0.25 ~ 0.5 )|
           ( -0.25 ~ 0.25 ~ 0.25 )|
           ( -0.25 ~ 0.25 ~ 0.25 )|
           
           ( -0.5 ~ 2 ~ 0.5 )|
           ( -3.5 ~ 0.5 ~ 0.5 )|
           ( -1 ~ 0.25 ~ 0.5 )|
           ( -0.25 ~ 1.0 ~ 0.5 );
           
graphset;
begwind;
margin(0,0,0.2,0.2);
window(nvar,nvar,0);
fonts("microb");
_ptitlht = 0.3;
_paxht = 0.3;
_pnumht = 0.25;
_pltype = {6 3 3 1};
_pcolor = {1 4 4 1};
_protate = 0; 
_plctrl = 0;
_plwidth = 6.5;
_psymsiz = 5;
_protate = 0;
_plegstr = "Money-Search\000MIU";
/* _plegctl = 1;  legend appears in the right-lower corner of page */ 
/* _plegctl = { 2 5 4.5 4.0 }; */

pl_ind = 1;
do until pl_ind > nvar*(nvar);

   title(titlestr[pl_ind]);
   ylabel(ystr);       
   /* ytics(drawtics[pl_ind,1],drawtics[pl_ind,2],drawtics[pl_ind,3],1); */
   xy(ti, dsgeirf1_m[.,pl_ind]~dsgeirf2_m[.,pl_ind]);  
   /* xy(ti, dsgeirf1_m[.,pl_ind]~dsgeirf1_hpd_l[.,pl_ind]~dsgeirf1_hpd_h[.,pl_ind]); */
   
   $titlestr[pl_ind];
   "";
   ti~dsgeirf1_m[.,pl_ind]~dsgeirf2_m[.,pl_ind];
   "";
   "";
   
   nextwind;
                 
   pl_ind = pl_ind+1;
endo;
    
endwind;
