/******************************************************************************
* DSGE_ACSAMP.G
* converts the DSGE parameters into unconditional sample moments
* 
*
* upadted:  03/11/2005
******************************************************************************/
new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  __output = 1;
  _fcmptol = 1E-10;
  _verbose = 1;           /** 1 for verbose **/

mhrun  = "1";

#include dsge_mod.src;
#include dsge_para.src;
#include dsge_spec.g;
#include pathspec.g;
#include dsge_sim.src;

/********************************************************
* Import data series
********************************************************/
#include loaddata.g;

nobs  = rows(YY);          /** number of observations **/
ny    = cols(YY);          /** number of series **/

nobs_burn  = 100;
nobs_total = nobs_burn + nobs;

/***********************************************************************
* Sample Stats: stdc(interest), stdc(velocity), corr(interest,velocity)
***********************************************************************/

corrYY    = corrx(YY[.,3]~YY[.,4]/100);
ac_data   = stdc(YY[.,3])
          ~ stdc(YY[.,4]/100)
          ~ corrYY[1,2];

nac = cols(ac_data);

/** Initialize Output files **/
opath  = resupath $+ "\\mhrun" $+ mhrun;
chkdir = chk_dir(opath);

/** Save actual sample moments **/
oac  = opath $+ "\\data_ac";
create fhac = ^oac with AC, nac, 8;
writer(fhac,ac_data);
closeall fhac;

/********************************************************************
* Load Parameter Draws from MH Output
********************************************************************/
nblock = 10000;   /* block size */
nuse   = 1000;    /* use every nuse observation */

lpath  = parapath $+ "\\mhrun" $+ mhrun;
opath  = resupath $+ "\\mhrun" $+ mhrun;
chkdir = chk_dir(opath);

lpara  = lpath $+ "\\pa";

open fhpara = ^lpara for read;

/** Initialize Output files **/
oac  = opath $+ "\\acsamp";
create fhac = ^oac with AC, nac, 8;

/** Initialization **/
eofloop = 0;
loopct  = 1;

if _verbose;
   "";
   "Calculating moments...";
   "----------------------";
endif;

do until eofloop;

   if _verbose;
      "";
      "Block           " ftos(loopct,"%*.*lf",6,0);
   endif;

   /** Read blocks of size nblock **/
   parasim = readr(fhpara,nblock);
   nread   = rows(parasim);

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .NE 0;

   parasim = delif(parasim,indseq);
   nread   = rows(parasim);

   if ismiss(parasim) == 0;

      acsim   = zeros(nread,nac);

      j = 1;
      do until j > nread;
         para = parasim[j,.]';
      
         ac = dsgesampmom(para,msel,nobs_burn,nobs_total);
         acsim[j,.] = ac;

         if _verbose;
           if fmod(j,nread/10)==0; ".";; pause(1e-20);endif;
         endif;
         j = j+1;
      endo;

      /** write Output **/
      wr = writer(fhac,acsim);
      
   endif;

   if _verbose;
      "";
      "Draws Processed " ftos(rows(acsim),"%*.*lf",6,0);
        "";
   endif;

   loopct = loopct + 1;
   eofloop = eof(fhpara);

endo;

closeall fhpara, fhac;

end;

/********************************************************************
* Procedure: DSGESAMPMOM
********************************************************************/
proc (1) = dsgesampmom(para,msel,nobs_burn,nobs_total);

local YYGsim, YYchisim, YYRsim, YYZsim, YYpisim, YYsim,
      ac_sim, corrYY;

{YYGsim, YYchisim, YYRsim, YYZsim, YYpisim, YYsim} 
   = dsgesim(para,msel,nobs_burn,nobs_total);

corrYY = corrx(YYpisim[.,3]~YYpisim[.,4]/100);

ac_sim = stdc(YYpisim[.,3])
       ~ stdc(YYpisim[.,4]/100)
       ~ corrYY[1,2];

retp(ac_sim);
endp;

