/********************************************************************
** Likelihood Evaluation using Kalman Filter
**
** {loglh,retcode,obserror,obsvar} = evalmod(para,YY,msel);
** #include ****_mod.src,  to run this procedure
**
** Last updated: 06/16/2005 
********************************************************************/

proc (1) = evalpenalty(para,msel,sstargets);
/* targets are: i/y, markup, */
local sst, valid, penalty, T0, T1, TC, TETA, RC, retcode;

/* steady states
** 33: ist
** 34: ycalst
** 45: markup-dm
** 46: markup 
** 43: laborshare 
*/

if msel[2] == 1;

   {T1,TC,T0,TETA,RC,retcode} = dsgesolv(para,msel);
   {sst, valid}=dsgess(para,msel); 

   if (valid == 0) or (retcode == 1) or (RC[1] NE 1) or (RC[2] NE 1); 
      penalty = -1E8;
   else;
      penalty = -0.5*((sst[33]/sst[34])-sstargets[1,1])^2/sstargets[1,2]^2 
                -0.5*(sst[43]-sstargets[3,1])^2/sstargets[3,2]^2
                -0.5*(sst[46]-sstargets[2,1])^2/sstargets[2,2]^2              
                -0.5*(sst[45]-sstargets[4,1])^2/sstargets[4,2]^2;
      /*pause(1e-10);*/
   endif;

else; 

   penalty = 0;
   
endif;

retp(penalty);
endp;


proc (4) = evalmod(para,YY,msel);

local loglh,retcode,obserror,obsvar,loglhzero,
      nobs,ny,nstate,nz,nep,tcond,
      TT,QQ,RR,HH,DD,ZZ,VV,RC,QQ2,QQt,
      t,At,Pt,At1,Pt1,Ft,alphahat,Phat,nut,yhat,
      Tqz, Sqz, Qqz, Zqz, lambdaqz, RRqz, nstate_rr, wtoalp, alptow,i,
      PItarget0, PItarget_t;

/** likelihood conditional on first tcond observations **/
/** to make it comparable to VAR. We do not use this here **/
tcond = 4;
  
PItarget0 = para[28]; 

/** initialize **/
nobs = rows(YY);
ny   = cols(YY);

loglhzero = -1E8;

loglh     = 0;
retcode   = 0;
obserror  = zeros(nobs,ny);
obsvar    = zeros(nobs,ny);


/**********************************************************
**      System matrices for State-space
**********************************************************/
{TT,QQ,RR,HH,DD,ZZ,VV,RC,QQ2}=sysmat(para,msel);

/**********************************************************
** Check determinacy 
**********************************************************/
if (RC[1] == 1) AND (RC[2]==1);
   /* determinacy */
   retcode[1] = 0;

elseif (RC[1] == 1) AND (RC[2]==0) ;
   /* indeterminacy */
   retcode[1] = 1;
   loglh = loglhzero;
   goto eofeval;

else;
   /* no equilibrium exists, numerical problems */
   retcode[1] = RC[1];
   loglh = loglhzero;
   goto eofeval;

endif;

/**********************************************************
**      Likelihood evaluation by Kalman filter
**********************************************************/
nstate = rows(TT);

/** initial mean and variance for the state vector **/
QQt = QQ;
At = zeros(nstate,1);
At[nstate,1] = PItarget0;
Pt = zeros(nstate,nstate);
Pt[1:nstate-1,1:nstate-1] = dlyap(TT[1:nstate-1,1:nstate-1],RR[1:nstate-1,.]*QQt*RR[1:nstate-1,.]');

Pitarget_t = zeros(nobs,1);

/** filter loop **/
t = 1;
do until t > nobs;

   At1 = At;
   Pt1 = Pt;
   
   if (t GE _tbegin2) AND (t LE _tend2);
      QQt = QQ2;
   else;
      QQt = QQ;
   endif;

   /** Forecasting **/
   alphahat = TT*At1;
   Phat     = TT*Pt1*TT' + RR*QQt*RR';
   Phat     = .5*(Phat+Phat');

   yhat     = ZZ*alphahat + DD;
   nut      = YY[t,.] - yhat';
   
   Ft       = ZZ*Phat*ZZ' + HH + ZZ*RR*VV + (ZZ*RR*VV)';
   Ft       = .5*(Ft+Ft');
      
   /** to make likelihood comparable to VAR(tcond), condition on first tcond observations **/
   if t > tcond;
    loglh    =  loglh -0.5*cols(YY)*ln(2*pi) - 0.5*ln(det(Ft)) - 0.5*nut*inv(Ft)*nut';
   endif;
  
   if sumc(eig(Ft).<0) > 0;
      /* numerical problem */
      retcode[1] = -5;
      loglh = loglhzero;
      goto eofeval;
   endif;

   /** Updating **/
   At = alphahat + Phat*ZZ'*inv(Ft)*nut';
   Pt = Phat - (Phat*ZZ'+RR*VV)*inv(Ft)*(Phat*ZZ'+RR*VV)';
   Pt = .5*(Pt+Pt');

   /* PItarget_t[t] = At[nstate]+DD[2,1];  */
   /* PItarget_t[t] = At[49]; */

   obserror[t,.] = nut;
   obsvar[t,.]   = diag(Ft)';

   t = t+1;
endo;

/* xy(seqa(1964,0.25,rows(YY)),PItarget_t~YY[.,2]); */
/*
graphset;
xy( seqa(1964,0.25,rows(YY)), PItarget_t )  ; 
meanc(PItarget_t);
end;
*/

eofeval:

if ismiss(real(loglh))==1;
   loglh = loglhzero;
endif;

retp(real(loglh),retcode,obserror,obsvar);
endp;
