/* filename:    irfplot.g
** description: The program plots the 
**              the DSGE impulse response functions
** created:     11/06/01
*/


new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  _verbose = 1;           /** 1 for verbose **/

#include dsge_spec.g;
#include pathspec.g;


/******************************************************************
**         Load IRF summaries
*/

ldsgeirf1 = workpath $+ "\\m1_t2m21t2p3\\results\\mhrun1\\dsgeirfs";
ldsgeirf2 = workpath $+ "\\m1_t2m2lt2p3\\results\\mhrun1\\dsgeirfs";

ti    = seqa(1,1,nirf);
nresp = 11;
nirfplot = 20;

/* Read Posterior Simulation Summary File:
** DSGE model 1
*/
open fhdsgeirf = ^ldsgeirf1 for read;

dsgeirf1_m = readr(fhdsgeirf,1);
dsgeirf1_s = readr(fhdsgeirf,1);
dsgeirf1_hpd_l = readr(fhdsgeirf,1);
dsgeirf1_hpd_h = readr(fhdsgeirf,1);

dsgeirf1_m     = reshape(dsgeirf1_m,nvar*nresp,nirf)';
dsgeirf1_hpd_l = reshape(dsgeirf1_hpd_l,nvar*nresp,nirf)';
dsgeirf1_hpd_h = reshape(dsgeirf1_hpd_h,nvar*nresp,nirf)';

closeall;

/* Read Posterior Simulation Summary File:
** DSGE model 2
*/
/*
open fhdsgeirf = ^ldsgeirf2 for read;

dsgeirf2_m = readr(fhdsgeirf,1);
dsgeirf2_s = readr(fhdsgeirf,1);
dsgeirf2_hpd_l = readr(fhdsgeirf,1);
dsgeirf2_hpd_h = readr(fhdsgeirf,1);

dsgeirf2_m     = reshape(dsgeirf2_m,nvar*nresp,nirf)';
dsgeirf2_hpd_l = reshape(dsgeirf2_hpd_l,nvar*nresp,nirf)';
dsgeirf2_hpd_h = reshape(dsgeirf2_hpd_h,nvar*nresp,nirf)';

closeall fhdsgeirf;
*/

/* Plot Impulse Response functions
*/

/* select shock
** 1 = government spending
** 2 = money demand
** 3 = policy (short run)
** 4 = technology
** 5 = policy (long run)
*/

shsel = 3;

titlestr = "Real GDP" $|
           "Agg. Inflation" $|
           "CM Output" $|
           "DM Output" $|
           "DM Inflation" $|
           "DM/CM Rel. Price" $|
           "Nom. Interest" $|
           "Real Money"    $|
           "Target Inflation" $|
           "CM Inflation" $|           
           "Inverse Velocity";
           
if shsel == 2;

   drawtics = ( -0.5 ~ 0.5 ~ 0.5 )|
              ( -1.0 ~ 1 ~ 1 )|
              ( -0.5 ~ 0.5 ~ 0.5 )|
              (  0   ~ 2   ~ 1   )|          
              ( -1.0 ~ 2 ~ 1 )|
              ( -1   ~ 1   ~ 1   )|
              ( -0.5 ~ 0.5 ~ 0.5 )|
              (  0   ~ 2  ~ 1   );

elseif shsel == 3;

   drawtics = ( -0.7 ~ 0.1 ~ 0.4 )|
              ( -0.6 ~ 0.2 ~ 0.2 )|
              ( -0.7 ~ 0.1 ~ 0.4 )|
              ( -0.7 ~ 0.1 ~ 0.4 )|          
              ( -2 ~ 1 ~ 1 )|
              ( -0.6 ~ 0.2 ~ 0.2 )|
              ( -0.2 ~ 0.8 ~ 0.5 )|
              ( -1.5 ~ 0.5 ~ 0.5 );

elseif shsel == 4;

   drawtics = (  0   ~ 1   ~ 0.5 )|
              ( -1.5 ~ 0.5 ~ 0.5 )|
              (  0   ~ 1   ~ 0.5 )|
              (  0   ~ 1   ~ 0.5 )|
              ( -4   ~ 2   ~ 2   )|
              ( -1.5 ~ 0.5 ~ 0.5 )|
              ( -0.3 ~ 0.1 ~ 0.2 )|
              ( 0 ~ 0.5 ~ 0.25 );

elseif shsel == 5;

   drawtics = ( -0.25   ~ 0.25 ~ 0.25 )|
              (  0   ~ 0.5 ~ 0.25 )|
              ( -0.25   ~ 0.25 ~ 0.25 )|
              ( -0.25   ~ 0.25 ~ 0.25 )|          
              (  0   ~ 0.5 ~ 0.25 )|
              ( -0.25 ~ 0.25 ~ 0.25 )|
              (  0   ~ 0.5 ~ 0.25 )|
              ( -0.25 ~ 0.25 ~ 0.25 ) ;

endif;

           
graphset;
begwind;
margin(0,0,0.2,0.2);
window(4,2,0);
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.3;
_pnumht = 0.4;
_pltype = {6 3 3 1};
_pcolor = {1 4 4 1};
_protate = 0; 
_plctrl = 0;
_plwidth = 6.5;
_psymsiz = 5;
_protate = 0;
/* _plegstr = "nu=0.2\000nu=1.0"; */

_pline = ( 1 ~ 6 ~ 0 ~ 0 ~ maxc(ti[1:nirfplot]) ~ 0 ~ 4 ~ 2);

pl_ind = 1;
do until pl_ind > 8;

   irfindx = (shsel-1)*nresp;

   title(titlestr[pl_ind]);
   ytics(drawtics[pl_ind,1],drawtics[pl_ind,2],drawtics[pl_ind,3],2); 

   xy(ti[1:nirfplot], dsgeirf1_m[1:nirfplot,irfindx+pl_ind]
         ~ dsgeirf1_hpd_l[1:nirfplot,irfindx+pl_ind]
         ~ dsgeirf1_hpd_h[1:nirfplot,irfindx+pl_ind] );

   nextwind;
   
   /*
   $titlestr[pl_ind];
   ti~dsgeirf1_m[.,pl_ind]~dsgeirf1_hpd_l[.,pl_ind]~dsgeirf1_hpd_h[.,pl_ind];
   "==================";
   "";
   */
               
   pl_ind = pl_ind+1;
endo;
    
endwind;
