/******************************************************************************
* NKMP_PM.G
* maximizes posterior density
*
* upadted:  06/02/2005
******************************************************************************/
new;

/** set seed **/
/** random number generator used in optimization procedure **/
/* rndseed 1234; */


/* this procedure can be used for the posterior
** and for a penalty function based prior
*/

_priorsim = 0; 
optindx = "1"; 

if _priorsim == 1;
   priorstr = "prior";
else;
   priorstr = "";
endif;

/********************************************************
* House cleaning and set defaults
********************************************************/
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
__output = 1;       /** 1 for verbose of CSMIN**/
__graph  = 1;       /** 1 for graph mode **/
_verbose = 2;       /** 0 for silence, 1 for compact, 2 for detailed**/
_fcmptol = 1E-10;

/** only the following 2 lines are model dependent **/
#include dsge_mod.src;
#include dsge_para.src;
#include dsge_spec.g;
#include evalmod.g
#include pathspec.g;

/********************************************************
*  Choose Model Parameters
********************************************************/

goto m1m11t2p3para;

m1m11t2p3para:

del   = 0.0140; /* depreciation rate */
alp   = 0.2820; /* capital share     */
lam   = 0.1653; /* markup - (1+lam)/lam is elasticity of production of final */
                /*          good Y_t wrt intermeadite good Y(i)_t            */
gam   = 1.0000; /* risk aversion parameter in HH utility B*X_t^(1-gam)/(1-gam) */

chist = 1.0000;
zet   = 0.7557;   /* (1-zet) prob of reoptimizing prices by firms */
gst   = 1.2000;   /* (1-1/gst) is govt spending share of GDP in ss */
FF    = 0.0000;   /* fixed costs in production of intermediate goods */

lnmyp = -0.3800;    /* inverse velocity  */
nu    = 31.7539;   /* elasticity parameter for money demand fcn */
iota  = 0.0357;    /* fraction of firms that use dynamic indexation */
psi1  = 1.7000;    /* response of int rate to inflation in MP rule */
psi2  = 1.0271;    /* response of int rate to real GDP in MP rule */
 
rho_gov = 0.8960;  /* persistence: govt expenditures process */
rho_chi = 0.9824;  /* persistence: preference shock */
rho_r   = 0.6688;  /* persistence of int rate in MP rule */
rho_z   = 0.7992;  /* persistence: technology shock */

pi_steady    = 4.000;  /* steady state inflation */
rr_steady    = 2.500;  /* steady state real rate */
pi_steady_st = 0.000;  /* static indexation of firms that don't reoptimize prices */
      
sigGOV = 1.1405;    /* Stdev: shock to govt expenditures process */
sigCHI = 1.2978;    /* Stdev: preference shock */
sigR   = 0.3376;    /* Stdev: monetary policy shock */
sigZ   = 2.0815;    /* Stdev: technology shock */
sigPi  = 0.0487;    /* Stdev: inflation target shock */

Spp    = 5.2850;   /* investment adjustment cost parameter */

psi2   = 0.0;
Spp    = 0.001;


lnhstyst = -3.35;
lnyst  = 0;
PItarget0 = -0.0582;
sigR2  = 0.8096;

goto endpara;


endpara:

para =   
      del|alp|lam|gam| chist|zet|gst|FF|lnmyp|nu|iota|psi1|psi2| 
      rho_gov|rho_chi| rho_r| rho_z|      pi_steady| rr_steady| pi_steady_st|
      sigGOV| sigCHI| sigR| sigZ | sigPI | Spp| lnhstyst| lnyst |PItarget0 | sigR2;

npara = rows(para);

/********************************************************
* Import data series
********************************************************/
#include loaddata.g;

nobs  = rows(YY);          /** number of observations **/
ny    = cols(YY);          /** number of series **/

/********************************************************
* Calculate historical decomposition at para
********************************************************/

/* shocks: g=1; chi=2; R=3; Z=4; pi=5 */
shind = 2;

{lnpY,retcode,YYhist} = histdecomp(para,YY,msel,shind);

    /** titlestr is defined in ****_spec.g **/


titlestr = "Output" $|
           "Inflation" $|
           "Interest Rate" $|
           "Inverse Velocity";

graphset;
begwind;
margin(0,0,0.2,0.2);
window(2,2,0);
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.3;
_pnumht = 0.4;

_protate = 0; 
_plctrl = 0;
_plwidth = 7.5;
_psymsiz = 5;
_protate = 0;

_pltype = {6 3 };
_pcolor = {1 4 };

yticmat = (-10 ~ 10 ~ 10) |
          ( 0  ~ 12 ~ 4) |
          ( 0  ~ 18 ~ 6) |
          (-80 ~ 0 ~  20);






ser_ind = 1;
do until ser_ind > 4;

      /* ylabel(ystr[1]); */
      title(titlestr[ser_ind]);
      ytics(yticmat[ser_ind,1],yticmat[ser_ind,2],yticmat[ser_ind,3],2); 
      xy(ti,YY[.,ser_ind]~YYhist[.,ser_ind]);
      ser_ind = ser_ind+1;
      nextwind;

endo;

endwind;
    
end;


/**********************************************************
**      P R O C E D U R E S
**********************************************************/

proc (3) = histdecomp(para,YY,msel,shind);

local loglh,retcode,RRsh,YYhist,Athist,loglhzero,
      nobs,ny,nstate,nz,nep,tcond,
      TT,QQ,RR,HH,DD,ZZ,VV,RC,QQ2,QQt,
      t,At,Pt,At1,Pt1,Ft,alphahat,Phat,nut,yhat,
      Tqz, Sqz, Qqz, Zqz, lambdaqz, RRqz, nstate_rr, wtoalp, alptow,i,
      PItarget0, PItarget_t;

/** likelihood conditional on first tcond observations **/
/** to make it comparable to VAR. We do not use this here **/
tcond = 4;

PItarget0 = para[29]; 

  
/** initialize **/
nobs = rows(YY);
ny   = cols(YY);

loglhzero = -1E8;

loglh     = 0;
retcode   = 0;
YYhist    = zeros(nobs,ny);

/**********************************************************
**      System matrices for State-space
**********************************************************/
{TT,QQ,RR,HH,DD,ZZ,VV,RC,QQ2}=sysmat(para,msel);

/**********************************************************
** Check determinacy 
**********************************************************/
if (RC[1] == 1) and (RC[2]==1);
   /* determinacy */
   retcode[1] = 0;

elseif (RC[1] == 1) and (RC[2]==0) ;
   /* indeterminacy */
   retcode[1] = 1;
   loglh = loglhzero;
   goto eofeval;

else;
   /* no equilibrium exists, numerical problems */
   retcode[1] = RC[1];
   loglh = loglhzero;
   goto eofeval;

endif;

/**********************************************************
**      Kalman filter iterations
**********************************************************/

/* augment the state vector by shock,
** shock comes first, pistarget remains last
*/
nstate = rows(TT);
TT     = zeros(1,nstate+1) |
        ( zeros(nstate,1) ~ TT );

RRsh          = zeros(1,cols(RR));
RRsh[1,shind] = 1;     
RR            = RRsh | RR ;

/* adjust the measurement equation
*/
ZZ = zeros(rows(ZZ),1) ~ ZZ;

nstate = nstate + 1;

/** initial mean and variance for the state vector **/
QQt = QQ;
At = zeros(nstate,1);
At[nstate,1] = PItarget0;
Pt = zeros(nstate,nstate);
Pt[1:nstate-1,1:nstate-1] = dlyap(TT[1:nstate-1,1:nstate-1],RR[1:nstate-1,.]*QQt*RR[1:nstate-1,.]');

/** filter loop **/
t = 1;
do until t > nobs;

   At1 = At;
   Pt1 = Pt;
   
   if (t ge _tbegin2) and (t le _tend2);
      QQt = QQ2;
   else;
      QQt = QQ;
   endif;

   /** Forecasting **/
   alphahat = TT*At1;
   Phat     = TT*Pt1*TT' + RR*QQt*RR';
   Phat     = .5*(Phat+Phat');

   yhat     = ZZ*alphahat + DD;
   nut      = YY[t,.] - yhat';
   
   Ft       = ZZ*Phat*ZZ' + HH + ZZ*RR*VV + (ZZ*RR*VV)';
   Ft       = .5*(Ft+Ft');
      
   /** to make likelihood comparable to VAR(tcond), condition on first tcond observations **/
   if t > tcond;
    loglh    =  loglh -0.5*cols(YY)*ln(2*pi) - 0.5*ln(det(Ft)) - 0.5*nut*inv(Ft)*nut';
   endif;
  
   if sumc(eig(Ft).<0) > 0;
      /* numerical problem */
      retcode[1] = -5;
      loglh = loglhzero;
      goto eofeval;
   endif;

   /** Updating **/
   At = alphahat + Phat*ZZ'*inv(Ft)*nut';
   Pt = Phat - (Phat*ZZ'+RR*VV)*inv(Ft)*(Phat*ZZ'+RR*VV)';
   Pt = .5*(Pt+Pt');

   if t > tcond;
      Athist      = TT*Athist + RR*RRsh'*At[1];
      YYhist[t,.] = (ZZ*Athist + DD)';

   else;
      YYhist[t,.] = YY[t,.];
      Athist      = At;
   endif;

   t = t+1;
endo;

eofeval:

if ismiss(real(loglh))==1;
   loglh = loglhzero;
endif;

retp(real(loglh),retcode,YYhist);
endp;
