/******************************************************************************
* DSGE_SS.G
* for every draw of DSGE parameters, computes steady state values of variables 
*
* created:  03-SEP-2007
* based on: dsge_ac.g
******************************************************************************/
new;
closeall;
library pgraph, user;
dlibrary -a PszTgSen,PsdTgSen;

cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

#include dsge_spec.g;
#include pathspec.g;
#include dsge_mod.src;
msel = 1 | 1;

/********************************************************************
* Specify parameter values
********************************************************************/

goto m1m11t2p2para;

/* Baseline MIU Model 
** posterior mean based on M1 data 
*/             
m1m11t2p2para:

del   = 0.0140; /* depreciation rate */
alp   = 0.2827; /* capital share     */
lam   = 0.1649; /* markup - (1+lam)/lam is elasticity of production of final */
                /*          good Y_t wrt intermeadite good Y(i)_t            */
gam   = 1.0000; /* risk aversion parameter in HH utility B*X_t^(1-gam)/(1-gam) */

chist = 1.0000;
zet   = 0.7384;   /* (1-zet) prob of reoptimizing prices by firms */
gst   = 1.2000;   /* (1-1/gst) is govt spending share of GDP in ss */
FF    = 0.0000;   /* fixed costs in production of intermediate goods */

lnmyp = -0.4358;    /* inverse velocity  */
nu    = 31.3563;   /* elasticity parameter for money demand fcn */
iota  = 0.0326;    /* fraction of firms that use dynamic indexation */
psi1  = 2.0000;    /* response of int rate to inflation in MP rule */
psi2  = 0.13852;    /* response of int rate to real GDP in MP rule */
 
rho_gov = 0.8836;  /* persistence: govt expenditures process */
rho_chi = 0.9800;  /* persistence: preference shock */
rho_r   = 0.6964;  /* persistence of int rate in MP rule */
rho_z   = 0.9800;  /* persistence: technology shock */

pi_steady    = 4.000;  /* steady state inflation */
rr_steady    = 2.500;  /* steady state real rate */
pi_steady_st = 0.000;  /* static indexation of firms that don't reoptimize prices */
      
sigGOV = 1.1746;    /* Stdev: shock to govt expenditures process */
sigCHI = 1.3071;    /* Stdev: preference shock */
sigR   = 0.3695;    /* Stdev: monetary policy shock */
sigZ   = 1.3229;    /* Stdev: technology shock */
sigPi  = 0.0487;    /* Stdev: inflation target shock */

Spp    = 4.7361;   /* investment adjustment cost parameter */
lnhstyst = -3.35;
lnyst  = 0;
PItarget0 = -0.0036;
sigR2  = 0.8637;

goto endpara;


m1m11t2p3para:

del   = 0.0140; /* depreciation rate */
alp   = 0.2820; /* capital share     */
lam   = 0.1653; /* markup - (1+lam)/lam is elasticity of production of final */
                /*          good Y_t wrt intermeadite good Y(i)_t            */
gam   = 1.0000; /* risk aversion parameter in HH utility B*X_t^(1-gam)/(1-gam) */

chist = 1.0000;
zet   = 0.7557;   /* (1-zet) prob of reoptimizing prices by firms */
gst   = 1.2000;   /* (1-1/gst) is govt spending share of GDP in ss */
FF    = 0.0000;   /* fixed costs in production of intermediate goods */

lnmyp = -0.3800;    /* inverse velocity  */
nu    = 31.7539;   /* elasticity parameter for money demand fcn */
iota  = 0.0357;    /* fraction of firms that use dynamic indexation */
psi1  = 1.7000;    /* response of int rate to inflation in MP rule */
psi2  = 1.0271;    /* response of int rate to real GDP in MP rule */
 
rho_gov = 0.8960;  /* persistence: govt expenditures process */
rho_chi = 0.9824;  /* persistence: preference shock */
rho_r   = 0.6688;  /* persistence of int rate in MP rule */
rho_z   = 0.7992;  /* persistence: technology shock */

pi_steady    = 4.000;  /* steady state inflation */
rr_steady    = 2.500;  /* steady state real rate */
pi_steady_st = 0.000;  /* static indexation of firms that don't reoptimize prices */
      
sigGOV = 1.1405;    /* Stdev: shock to govt expenditures process */
sigCHI = 1.2978;    /* Stdev: preference shock */
sigR   = 0.3376;    /* Stdev: monetary policy shock */
sigZ   = 2.0815;    /* Stdev: technology shock */
sigPi  = 0.0487;    /* Stdev: inflation target shock */

Spp    = 5.2850;   /* investment adjustment cost parameter */

lnhstyst = -3.35;
lnyst  = 0;
PItarget0 = -0.0582;
sigR2  = 0.8096;

goto endpara;


m1m11t2p4para:

del   = 0.0140; /* depreciation rate */
alp   = 0.2819; /* capital share     */
lam   = 0.1646; /* markup - (1+lam)/lam is elasticity of production of final */
                /*          good Y_t wrt intermeadite good Y(i)_t            */
gam   = 1.0000; /* risk aversion parameter in HH utility B*X_t^(1-gam)/(1-gam) */

chist = 1.0000;
zet   = 0.7501;   /* (1-zet) prob of reoptimizing prices by firms */
gst   = 1.2000;   /* (1-1/gst) is govt spending share of GDP in ss */
FF    = 0.0000;   /* fixed costs in production of intermediate goods */

lnmyp = -0.3800;    /* inverse velocity  */
nu    = 5.1670;   /* elasticity parameter for money demand fcn */
iota  = 0.0389;    /* fraction of firms that use dynamic indexation */
psi1  = 1.7000;    /* response of int rate to inflation in MP rule */
psi2  = 1.0244;    /* response of int rate to real GDP in MP rule */
 
rho_gov = 0.8841;  /* persistence: govt expenditures process */
rho_chi = 0.9537;  /* persistence: preference shock */
rho_r   = 0.6576;  /* persistence of int rate in MP rule */
rho_z   = 0.8229;  /* persistence: technology shock */

pi_steady    = 4.000;  /* steady state inflation */
rr_steady    = 2.500;  /* steady state real rate */
pi_steady_st = 0.000;  /* static indexation of firms that don't reoptimize prices */
      
sigGOV = 1.0950;    /* Stdev: shock to govt expenditures process */
sigCHI = 3.2793;    /* Stdev: preference shock */
sigR   = 0.3461;    /* Stdev: monetary policy shock */
sigZ   = 1.9273;    /* Stdev: technology shock */
sigPi  = 0.0485;    /* Stdev: inflation target shock */

Spp    = 4.9876;   /* investment adjustment cost parameter */
lnhstyst = -3.35;
lnyst  = 0;
PItarget0 = -0.0140;
sigR2  = 0.8305;

goto endpara;

endpara:

/* Decision rules at zero inflation */
/* pi_steady_st = 0;  */

para =   
      del|alp|lam|gam| chist|zet|gst|FF|lnmyp|nu|iota|psi1|psi2| 
      rho_gov|rho_chi| rho_r| rho_z|      pi_steady| rr_steady| pi_steady_st|
      sigGOV| sigCHI| sigR| sigZ | sigPI | Spp| lnhstyst| lnyst |PItarget0 | sigR2;


/********************************************************************
* Compute steady states
********************************************************************/

{sst, valid} = dsgess(para,msel);

/** Report Results **/
"";
"Steady State of   |    Value ";
outmat = sst_names(msel)' $~ ftocv(sst,5,5) ;
$outmat;

/*
dsgeirf( para,msel,40,(0|0|0|1|0) );
*/


/********************************************************************
* Check for determinacy
********************************************************************/

{TT,QQ,RR,HH,DD,ZZ,VV,RC,QQ2}=sysmat(para,msel);

if (RC[1] == 1) and (RC[2]==1);
   "";
   "Determinacy";
else;
   "";
   "No Determinacy";
   "Return Code" RC';
   end;
endif;

TT;
end;

goto simdata;


/********************************************************************
* Compute autocovariances
********************************************************************/

{vd,ac} = dsgemom(para,msel);

let string draw_names = {
               "MU_GDPR", "MU_INFL", "MU_FFUNDS", "MU_M2R",
               
               "V_GDPR", "C0_GDPR_INFL", "C0_GDPR_FFUNDS", "C0_GDPR_M2R",
               "C0_INFL_GDPR", "V_INFL", "C0_INFL_FFUNDS", "C0_INFL_M2R",
               "C0_FFUNDS_GDPR", "C0_FFUNDS_INFL", "V_FFUNDS", "C0_FFUNDS_M2R",
               "C0_M2R_GDPR", "C0_M2R_INFL", "C0_M2R_FFUNDS", "V_M2R",
               
               "C1_GDPR", "C1_GDPR_INFL", "C1_GDPR_FFUNDS", "C1_GDPR_M2R",
               "C1_INFL_GDPR", "C1_INFL", "C1_INFL_FFUNDS", "C1_INFL_M2R",
               "C1_FFUNDS_GDPR", "C1_FFUNDS_INFL", "C1_FFUNDS", "C1_FFUNDS_M2R",
               "C1_M2R_GDPR", "C1_M2R_INFL", "C1_M2R_FFUNDS", "C1_M2R"
               };
             

draw_names = vec(draw_names)';

"";
"Autocovariance      |   Value";
outmat = draw_names' $~ ftocv(ac',4,4) ;
outmat;

let string draw_names = {
               "GDPR_GOV",   "GDPR_CHI",   "GDPR_R",   "GDPR_Z",
               "INFL_GOV",   "INFL_CHI",   "INFL_R",   "INFL_Z",
               "FFUNDS_GOV", "FFUNDS_CHI", "FFUNDS_R", "FFUNDS_Z",
               "M2R_GOV",    "M2R_CHI",    "M2R_R",    "M2R_Z"
             };
draw_names = vec(draw_names)';

"";
"Variance Decomposition |   Value";
outmat = draw_names' $~ ftocv(vec(vd'),4,4) ;
outmat;

/********************************************************************
* Simulate Data
********************************************************************/
simdata:

#include loaddata.g;
nobs  = rows(YY);          /** number of observations **/
ny    = cols(YY);          /** number of series **/

{yTG,yTchi,yTR,yTZ,yTpi,yTall} = dsgesim(para,msel,100,10100);

yTnopi = yTG+yTchi+yTZ+yTR;
stdc(yTnopi[.,3]);
corr_pigdp = corrx(yTnopi[2:10000,3]~yTnopi[1:9999,3]);
corr_pigdp;

end;

graphset;

fonts("microb");
_ptitlht= 0.3;
_paxht  = 0.25;
_pnumht = 0.2;
_protate= 0;
_plctrl = -1;
_pltype = {6 2 3};

begwind;
margin(0.2,0.2,0.2,0.2);
window(3,2,0);  


/* Name the AXIS labels
*/
ylabel("log (Inverse Velocity)");
xlabel("Nominal Interest Rate (%)");
xtics(0, 15, 5,1);
ytics(-1,0,0.2,1);
/*
_pmsgstr = "D";
_pmsgctl =  (Tsamp[1] ~ Tsamp[2] ~ 0.35 ~ 0 ~ 1 ~ 4 ~ 1);
*/

title("Monetary Policy (Short-Run)");
_pcolor = 2;
_psymsiz = 1;
xy(yTR[.,3],yTR[.,4]/100);
_psymsiz = 3;
_pcolor = 4;
xy(yy[.,3],yy[.,4]/100);
nextwind;

title("Monetary Policy (Long-Run)");
_pcolor = 2;
_psymsiz = 1;
xy(yTpi[.,3],yTpi[.,4]/100);
_psymsiz = 3;
_pcolor = 4;
xy(yy[.,3],yy[.,4]/100);
nextwind;

title("Technology");
_pcolor = 2;
_psymsiz = 1;
xy(yTz[.,3],yTz[.,4]/100);
_psymsiz = 3;
_pcolor = 4;
xy(yy[.,3],yy[.,4]/100);
nextwind;

title("Government Spending");
_pcolor = 2;
_psymsiz = 1;
xy(yTg[.,3],yTg[.,4]/100);
_psymsiz = 3;
_pcolor = 4;
xy(yy[.,3],yy[.,4]/100);
nextwind;

title("Money Demand");
_pcolor = 2;
_psymsiz = 1;
xy(yTchi[.,3],yTchi[.,4]/100);
_psymsiz = 3;
_pcolor = 4;
xy(yy[.,3],yy[.,4]/100);
nextwind;

title("All Shocks");
_pcolor = 2;
_psymsiz = 1;
xy(yTall[.,3],yTall[.,4]/100);
_psymsiz = 3;
_pcolor = 4;
xy(yy[.,3],yy[.,4]/100);
endwind;


/* Estimate MD on simulated data
*/
nograph:

nysim = rows(yTall);

gdp      = yTall[3:nysim,1];
gdplag1  = yTall[2:nysim-1,1];
gdplag2  = yTall[1:nysim-2,1];

infl      = yTall[3:nysim,2];
infllag1  = yTall[2:nysim-1,2];
infllag2  = yTall[1:nysim-2,2];

ffunds      = yTall[3:nysim,3];
ffundslag1  = yTall[2:nysim-1,3];
ffundslag2  = yTall[1:nysim-2,3];
 
myp      = yTall[3:nysim,4];
myplag1  = yTall[2:nysim-1,4];
myplag2  = yTall[1:nysim-2,4];

mp      = myp + gdp;
mplag1  = myplag1 + gdplag1;
mplag2  = myplag2 + gdplag2;

intcpt  = ones(nysim-2,1);

"Model Data - Levels";
xx      = intcpt ~ gdp ~ ffunds ~ mplag1 ~ infl;
bethat = inv(xx'*xx)*xx'*mp;
coeff_names =    "incept"
              $| "gdp   " 
              $| "ffunds"
              $| "mp(1) "
              $| "infl  ";
outmat = coeff_names $~ ftocv(bethat,4,4) ;
$outmat;
"";
"";

"Model Data - Differences";
xx      = intcpt ~ (gdp-gdplag1) ~ (ffunds-ffundslag1) ~ (infl-infllag1);
bethat  = inv(xx'*xx)*xx'*(mp-mplag1);
coeff_names =    "incept   "
              $| "D(gdp)   " 
              $| "D(ffunds)"
              $| "D(infl)  ";
outmat = coeff_names $~ ftocv(bethat,4,4) ;
$outmat;
"";
"";

"Model Data - VECM";
xx      = intcpt ~ gdplag1 ~ ffundslag1 ~ mplag1 ~ (gdplag1-gdplag2) 
          ~ (ffundslag1-ffundslag2) ~ (mplag1-mplag2);
bethat = inv(xx'*xx)*xx'*(mp-mplag1);
coeff_names =    "incept    "
              $| "gdp(1)    "
              $| "ffunds(1) "
              $| "mp(1)     "
              $| "D1(gdp)   " 
              $| "D1(ffunds)"
              $| "D1(mp)    ";
outmat = coeff_names $~ ftocv(bethat,4,4) ;
$outmat;
"";
"";

"Model Data - Stock Watson";
xx      = intcpt ~ gdp ~ ffunds ~ (gdplag1-gdplag2) 
          ~ (ffundslag1-ffundslag2);
bethat = inv(xx'*xx)*xx'*mp;
coeff_names =    "incept    "
              $| "gdp       "
              $| "ffunds    "
              $| "D1(gdp)   " 
              $| "D1(ffunds)"
              ;
outmat = coeff_names $~ ftocv(bethat,4,4) ;
$outmat;
"";
"";

"Model Data - IV";
xx      = intcpt ~ gdp ~ ffunds;
zz      = intcpt ~ gdplag1 ~ ffundslag1;
bethat = inv(zz'*zz)*xx'*mp;
coeff_names =    "incept    "
              $| "gdp       "
              $| "ffunds    "
              ;
outmat = coeff_names $~ ftocv(bethat,4,4) ;
$outmat;
"";
"";


/* Estimate MD on actual data
*/

nysim = rows(yy);

gdp      = yy[3:nysim,1];
gdplag1  = yy[2:nysim-1,1];
gdplag2  = yy[1:nysim-2,1];

infl      = yy[3:nysim,2];
infllag1  = yy[2:nysim-1,2];
infllag2  = yy[1:nysim-2,2];

ffunds      = yy[3:nysim,3];
ffundslag1  = yy[2:nysim-1,3];
ffundslag2  = yy[1:nysim-2,3];

myp      = yy[3:nysim,4];
myplag1  = yy[2:nysim-1,4];
myplag2  = yy[1:nysim-2,4];

mp      = myp + gdp;
mplag1  = myplag1 + gdplag1;
mplag2  = myplag2 + gdplag2;

intcpt  = ones(nysim-2,1);

"Actual Data - Levels";
xx      = intcpt ~ gdp ~ ffunds ~ mplag1 ~ infl;
bethat = inv(xx'*xx)*xx'*mp;
coeff_names =    "incept"
              $| "gdp   " 
              $| "ffunds"
              $| "mp(1) "
              $| "infl  ";
outmat = coeff_names $~ ftocv(bethat,4,4) ;
$outmat;
"";
"";

"Actual Data - Differences";
xx      = intcpt ~ (gdp-gdplag1) ~ (ffunds-ffundslag1) ~ (infl-infllag1);
bethat  = inv(xx'*xx)*xx'*(mp-mplag1);
coeff_names =    "incept   "
              $| "D(gdp)   " 
              $| "D(ffunds)"
              $| "D(infl)  ";
outmat = coeff_names $~ ftocv(bethat,4,4) ;
$outmat;
"";
"";

"Actual Data - VECM";
xx      = intcpt ~ gdplag1 ~ ffundslag1 ~ mplag1 ~ (gdplag1-gdplag2) 
          ~ (ffundslag1-ffundslag2) ~ (mplag1-mplag2);
bethat = inv(xx'*xx)*xx'*(mp-mplag1);
coeff_names =    "incept    "
              $| "gdp(1)    "
              $| "ffunds(1) "
              $| "mp(1)     "
              $| "D1(gdp)   " 
              $| "D1(ffunds)"
              $| "D1(mp)    ";
outmat = coeff_names $~ ftocv(bethat,4,4) ;
$outmat;
"";
"";

"Actual Data - Stock Watson";
xx      = intcpt ~ gdp ~ ffunds ~ (gdplag1-gdplag2) 
          ~ (ffundslag1-ffundslag2);
bethat = inv(xx'*xx)*xx'*mp;
coeff_names =    "incept    "
              $| "gdp       "
              $| "ffunds    "
              $| "D1(gdp)   " 
              $| "D1(ffunds)"
              ;
outmat = coeff_names $~ ftocv(bethat,4,4) ;
$outmat;
"";
"";


end;


/********************************************************************
* Procedure: DSGEMOM
********************************************************************/
proc (2) = dsgemom(para,msel);

local TT,QQ,QQ2,RR,HH,DD,ZZ,VV,RC,sig_chol,i,ny,
      GAM0s,GAM1s,GAM0i,GAM1i,GAM0,GAM1,QQi,GAM0tr,seldiag,seloffdiag,
      vd,ac;

/**********************************************************
**      System matrices for State-space
**********************************************************/
{TT,QQ,RR,HH,DD,ZZ,VV,RC,QQ2}=sysmat(para,msel);
ny = rows(ZZ);

/**********************************************************
**   variance decomposition and autocovariance
**********************************************************/

/** sqrt decomposition **/
sig_chol = sqrt(QQ);

GAM0 = zeros(ny,ny);
GAM1 = zeros(ny,ny);
vd   = zeros(ny,cols(sig_chol));

i=1;
do until i > cols(sig_chol);

   QQi   = sig_chol[.,i]*sig_chol[.,i]';
   GAM0s = real(dlyap(TT,RR*QQi*RR'));
   GAM0i = ZZ*GAM0s*ZZ' + ZZ*RR*VV + (ZZ*RR*VV)' + HH;
   GAM1i = ZZ*(TT*GAM0s)*ZZ' + ZZ*(TT*RR*VV);

   GAM0 = GAM0 + GAM0i;
   GAM1 = GAM1 + GAM1i;

   vd[.,i] = diag(GAM0i);

   i = i+1;
endo;

/** variance decomposition **/
vd = vd./diag(GAM0);

/** autocovariance **/

GAM0tr     = sqrt(diag(GAM0))*( sqrt(diag(GAM0)) )';
seldiag    = eye(rows(GAM0));
seloffdiag = ones(rows(GAM0),cols(GAM0)) - seldiag;
      
GAM0   = seldiag.*sqrt(GAM0tr) + seloffdiag.*( GAM0./GAM0tr);
GAM1   = GAM1./GAM0tr;

/** ny + ny^2 + ny^2 **/
ac = DD' ~ vec(GAM0)' ~ vec(GAM1)';

retp(vd,ac);
endp;

/********************************************************************
* Procedure: DSGESIM
********************************************************************/

proc (6) = dsgesim(para,msel,Tsimburn,Tsimtotal);
local TT,QQ,QQ2,RR,HH,DD,ZZ,VV,RC,
      HHchol,QQchol,neps,nstate,ny,
      ytG, ytchi, ytR, ytZ, ytpi,
      stG, stchi, stR, stZ, stpi,
      epst,t;

/* Compute system matrices for state space representation
*/
{TT,QQ,RR,HH,DD,ZZ,VV,RC,QQ2}=sysmat(para,msel);

neps   = cols(QQ);
nstate = cols(TT);
ny     = rows(ZZ);

QQchol = cholpsd(QQ);

/* Simulate DSGE model
*/

/* Initializations 
*/
yTG  = zeros(Tsimtotal+Tsimburn,ny);
yTchi= zeros(Tsimtotal+Tsimburn,ny);
yTR  = zeros(Tsimtotal+Tsimburn,ny);
yTZ  = zeros(Tsimtotal+Tsimburn,ny);
yTpi = zeros(Tsimtotal+Tsimburn,ny);

stG   = zeros(nstate,1);
stchi = zeros(nstate,1);
stR   = zeros(nstate,1);
stZ   = zeros(nstate,1);
stpi  = zeros(nstate,1);

/* Iterations
*/
t = 1;
do until t > Tsimburn + Tsimtotal;

   /* calculate realization of epst */
   epst   = QQchol*rndn(neps,1);
  
   /* Update vector x(t) */
   stG   = TT*stG   + RR*( (1|0|0|0|0) .* epst);
   stchi = TT*stchi + RR*( (0|1|0|0|0) .* epst);
   stR   = TT*stR   + RR*( (0|0|1|0|0) .* epst);
   stZ   = TT*stZ   + RR*( (0|0|0|1|0) .* epst);
   stpi  = TT*stpi  + RR*( (0|0|0|0|1) .* epst);
   
   yTG[t,.]   = (ZZ*stG)';
   yTchi[t,.] = (ZZ*stchi)';
   yTR[t,.]   = (ZZ*stR)';
   yTZ[t,.]   = (ZZ*stZ)';
   yTpi[t,.]  = (ZZ*stpi)';

   t = t+1;

endo;

yTG   = yTG[Tsimburn+1:Tsimtotal+Tsimburn,.];
yTchi = yTchi[Tsimburn+1:Tsimtotal+Tsimburn,.];
yTR   = yTR[Tsimburn+1:Tsimtotal+Tsimburn,.];
yTZ   = yTZ[Tsimburn+1:Tsimtotal+Tsimburn,.];
yTpi  = yTpi[Tsimburn+1:Tsimtotal+Tsimburn,.];
yTall = DD'+yTG+yTR+yTchi+yTz+yTpi;


retp(yTG+DD',yTchi+DD',yTR+DD',yTZ+DD',yTpi+DD',yTall);
endp;

/********************************************************************
**   Procedure to compute Impulse Response Function
********************************************************************/
proc (1) = dsgeirf(para,msel,nirf,imp);
local TT,QQ,RR,HH,DD,ZZ,VV,RC, ssirf, yyirf, t;

/**********************************************************
**      System matrices for State-space
**********************************************************/

{TT,QQ,RR,HH,DD,ZZ,VV,RC,QQ2}=sysmat(para,msel);

/* manual adjustment: turn inverse velocity into real money
** balances:
*/
/*ZZ[4,17]=0;*/


ssirf = zeros(nirf,cols(ZZ));
yyirf = zeros(nirf,rows(ZZ));

ssirf[1,.] = (RR*imp)';
yyirf[1,.] = ssirf[1,.]*ZZ';
t = 2;
do until t > nirf;
   ssirf[t,.] = (TT*ssirf[t-1,.]')';
   yyirf[t,.] = ssirf[t,.]*ZZ';
   t = t+1;
endo;

eofirf:

retp(yyirf);
endp;

