/******************************************************************************
* DSGE_SS.G
* for every draw of DSGE parameters, computes steady state values of variables 
*
* created:  03-SEP-2007
* based on: dsge_ac.g
******************************************************************************/
new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  __output = 1;
  _fcmptol = 1E-10;
  _verbose = 1;           /** 1 for verbose **/

#include dsge_mod.src;
#include dsge_para.src;
#include dsge_spec.g;
#include pathspec.g;


/********************************************************************
* Load Parameter Draws from MH Output
********************************************************************/
mhrun  = "1";
nblock = 10000;    /* block size */
nuse   = 10;      /* use every nuse observation */

lpath  = parapath $+ "\\mhrun" $+ mhrun;
opath  = resupath $+ "\\mhrun" $+ mhrun;
chkdir = chk_dir(opath);

lpara  = lpath $+ "\\pa";

open fhpara = ^lpara for read;

/** check dimensions **/
para    = pmsv(msel);
{sst, valid}=dsgess(para,msel);
/* {vd,ac} = dsgemom(para,msel); */

n_sst = rows(sst);

/** Initialize Output files **/
osst  = opath $+ "\\sst";

create fhsst = ^osst with SST, n_sst, 8;

/** Initialization **/
eofloop = 0;
loopct  = 1;

if _verbose;
   "";
   "Calculating steady states...";
   "----------------------------";
endif;

do until eofloop;

   if _verbose;
      "";
      "Block           " ftos(loopct,"%*.*lf",6,0);
   endif;

   /** Read blocks of size nblock **/
   parasim = readr(fhpara,nblock);
   nread   = rows(parasim);

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .NE 0;

   parasim = delif(parasim,indseq);
   nread   = rows(parasim);

if ismiss(parasim) == 0;

   sstsim     = zeros(nread, n_sst);
   /* acsim   = zeros(nread,nac);
   vdsim   = zeros(nread,nvd); */

   j = 1;
   do until j > nread;
      para = parasim[j,.]';
      
      /* {vd,ac} = dsgemom(para,msel); */
      {sst, valid} = dsgess(para,msel);

      /* Steady states */
      sstsim[j,.] =  sst';

      if _verbose;
         if fmod(j,nread/10)==0; ".";; pause(1e-20);endif;
      endif;
      j = j+1;
   endo;

    /** write Output **/
   wr = writer(fhsst,sstsim);

endif;

   if _verbose;
      "";
      "Draws Processed " ftos(rows(sstsim),"%*.*lf",6,0);
        "";
   endif;

   loopct = loopct + 1;
   eofloop = eof(fhpara);

endo;

closeall fhpara, fhsst;

end;


