/* filename:    irfplot.g
** description: The program plots the 
**              the DSGE impulse response functions
** created:     11/06/01
*/


new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  _verbose = 1;           /** 1 for verbose **/

#include vm_spec.g
#include pathspec.g;


/******************************************************************
**         Load IRF summaries
*/

lvarirf   = workpath $+    "\\m1_t2vmt2p1\\results\\mhrun1\\varirfs";

nresp = 11;
nvar  = 5;
nirf  = 40;
nirfplot = 20;
ti    = seqa(1,1,nirf);

/* Read Posterior Simulation Summary File:
** VAR: 
** Variables: "Real GDP", "Inflation", "Nom. Interest", "Real Money", "Target Inflation"
**
** Shocks: 
** 1 = policy (long run)
*/

open fhvarirf = ^lvarirf for read;

varirf_m     = readr(fhvarirf,1);
varirf_s     = readr(fhvarirf,1);
varirf_hpd_l = readr(fhvarirf,1);
varirf_hpd_h = readr(fhvarirf,1);

varirf_m     = reshape(varirf_m,nvar,nirf)';
varirf_hpd_l = reshape(varirf_hpd_l,nvar,nirf)';
varirf_hpd_h = reshape(varirf_hpd_h,nvar,nirf)';

/* eliminate target inflation response
*/
/*
varirf_m     = varirf_m[.,1:nvar-1];
varirf_hpd_l = varirf_hpd_l[.,1:nvar-1];
varirf_hpd_h = varirf_hpd_h[.,1:nvar-1];
*/

closeall;

/* Plot Impulse Response functions
*/

titlestr = "Real GDP" $|
           "Inflation" $|
           "Nom. Interest" $|
           "Inverse Velocity" $|
           "Target Inflation" ;

drawtics = ( -0.5 ~ 0.5 ~ 0.5 )  |
           (  0   ~ 0.5 ~ 0.25  )  |
           (  -0.5 ~ 0.5 ~ 0.5  )  |
           ( -2   ~ 1   ~ 1     )  |
           (  0   ~ 0.5 ~ 0.25   )  ;
          
graphset;
begwind;
margin(0,0,0.2,0.2);
window(1,2,0);
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.3;
_pnumht = 0.4;

_pltype = {6 3 3};
_pcolor = {1 4 4};

/*
_pltype = {6 1 3 3};
_pcolor = {1 6 4 4};
*/

/*
_pltype = {6 6 3 3};
_pcolor = {1 1 4 4};
*/
_protate = 0; 
_plctrl = 0;
_plwidth = 6.5;
_psymsiz = 5;
_protate = 0;
/* _plegstr = "DSGE\000VAR"; */

pl_ind = 2;
do until pl_ind > 5;

   title(titlestr[pl_ind]);
   ytics(drawtics[pl_ind,1],drawtics[pl_ind,2],drawtics[pl_ind,3],1); 
   xtics(0,20,5,5);

   /*
   xy(ti[1:nirfplot],  dsge1irf_m[1:nirfplot,pl_ind] ~ dsge2irf_m[1:nirfplot,pl_ind]
         ~ varirf_hpd_l[1:nirfplot,pl_ind]  ~ varirf_hpd_h[1:nirfplot,pl_ind] );
   */

   xy(ti[1:nirfplot],  varirf_m[1:nirfplot,pl_ind] 
         ~ varirf_hpd_l[1:nirfplot,pl_ind]  ~ varirf_hpd_h[1:nirfplot,pl_ind] );

         
   nextwind;
   
   /*
   $titlestr[pl_ind];
   ti~dsgeirf1_m[.,pl_ind]~dsgeirf1_hpd_l[.,pl_ind]~dsgeirf1_hpd_h[.,pl_ind];
   "==================";
   "";
   */
               
   pl_ind = pl_ind+3;
endo;
    
endwind;
